import 'package:flutter/material.dart';

import 'routes/bottom_sheet_dialog_route.dart';
import 'routes/dialog_route.dart';

class RouteUtils {
  static const String dialogRouteName = 'Dialog';

  static Route createPageRoute(Widget page) {
    return MaterialPageRoute(
        settings: _createRouteSettings(page),
        builder: (_) {
          return page;
        });
  }

  static Route createBottomSheetRoute(
      {Widget bottomSheet,
      bool isDismissible = true,
      bool enableDrag = true,
      bool isScrollControlled = false,
      Color backGroundColor = Colors.transparent}) {
    return BottomSheetDialogRoute(
        isDismissible: isDismissible,
        enableDrag: enableDrag,
        isScrollControlled: isScrollControlled,
        backgroundColor: backGroundColor,
        settings: _createRouteSettings(bottomSheet),
        builder: (_) {
          return bottomSheet;
        });
  }

  static Route createDialogRoute(
      {Widget dialog,
      bool isDismissible = true,
      Color backGroundColor = const Color(0x80000000)}) {
    return DialogRoute(
        barrierColor: backGroundColor,
        barrierDismissible: isDismissible,
        settings: _createRouteSettings(dialog),
        pageBuilder: (_, anim1, anim2) {
          return dialog;
        });
  }

  static RouteSettings _createRouteSettings(Widget page) {
    return RouteSettings(name: page.toStringShort());
  }
}

extension IsValidRouteExt on Route {
  bool isPage() {
    return !(this is DialogRoute) && !(this is BottomSheetDialogRoute);
  }
}
