import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

import "../core/exception/floative_exception.dart";
import '../core/flavor/flavor.dart';
import '../core/floative_base/floative.dart';
import '../floative_flutter.dart';
import 'route_utils.dart';

part 'observer/ship_navigation_observer.dart';
part 'ship_impl.dart';

abstract class Ship {
  GlobalKey<NavigatorState> navigationKey;

  NavigatorObserver navigationObserver;

  List<Route> pageStack;

  static Ship _instance;

  /// access to the Singleton instance of Ship
  static Ship get instance {
    _instance ??= _ShipImpl();
    return _instance;
  }

  /// Short form to access the instance of Ship
  static Ship get I => instance;

  factory Ship.newInstance() {
    return _ShipImpl();
  }

  void to({Widget page});

  void back({result});

  void replace({Widget page});

  void goBackTo({Widget page});

  Future<T> startPageForResult<T>({Widget page});

  Future<dynamic> showDialog(
      {Widget dialog,
      bool isDismissible = true,
      Color backGroundColor = const Color(0x80000000)});

  Future<dynamic> showBottomSheet(
      {Widget bottomSheet,
      bool isDismissible = true,
      bool enableDrag = true,
      bool isScrollControlled = false,
      Color backGroundColor = Colors.transparent});

  void debugShowPages();

  void init();

  void _onPop(Route route, Route previousRoute);
  void _onPush(Route route, Route previousRoute);
  void _onReplace(Route newRoute, Route oldRoute);
}
