part of 'ship.dart';

class _ShipImpl implements Ship {
  @override
  List<Route> pageStack = <Route>[];

  @override
  GlobalKey<NavigatorState> navigationKey = GlobalKey<NavigatorState>();

  @override
  NavigatorObserver navigationObserver = _ShipNavigationObserver();

  @override
  void init() {
    navigationObserver = _ShipNavigationObserver(
        onPush: _onPush, onPop: _onPop, onReplace: _onReplace);
  }

  @override
  void to({Widget page}) {
    navigationKey.currentState.push(RouteUtils.createPageRoute(page));
  }

  @override
  void back({dynamic result}) {
    navigationKey.currentState.pop(result);
  }

  @override
  void replace({Widget page}) {
    navigationKey.currentState
        .pushReplacement(RouteUtils.createPageRoute(page));
  }

  @override
  void goBackTo({Widget page}) {
    while (pageStack.last != RouteUtils.createPageRoute(page)) {
      navigationKey.currentState.pop();
    }
  }

  @override
  Future<T> startPageForResult<T>({Widget page}) async {
    var result =
        await navigationKey.currentState.push(RouteUtils.createPageRoute(page));

    if (result.runtimeType != T) {
      throw FloativeException("The received result type: ${result.runtimeType}"
          " does not match the expected type of $T");
    } else {
      return result;
    }
  }

  @override
  Future<dynamic> showBottomSheet(
      {Widget bottomSheet,
      bool isDismissible = true,
      bool enableDrag = true,
      bool isScrollControlled = false,
      Color backGroundColor = Colors.transparent}) async {
    var result = await navigationKey.currentState.push(
        RouteUtils.createBottomSheetRoute(
            isDismissible: isDismissible,
            enableDrag: enableDrag,
            isScrollControlled: isScrollControlled,
            backGroundColor: backGroundColor,
            bottomSheet: bottomSheet));

    return result;
  }

  @override
  Future<dynamic> showDialog({
    Widget dialog,
    bool isDismissible = true,
    Color backGroundColor = const Color(0x80000000),
  }) async {
    var result =
        await navigationKey.currentState.push(RouteUtils.createDialogRoute(
            isDismissible: isDismissible,
            backGroundColor: backGroundColor,
            dialog: Dialog(
              child: dialog,
            )));

    return result;
  }

  @override
  void debugShowPages() {
    if (Floative.I.currentFlavor != Flavor.development) {
      return;
    } else {
      if (pageStack.isEmpty) return;

      debugPrint("┌───────────────────────────────────────");

      debugPrint("| Stack of all living pages: ");

      for (var index = 0; index < pageStack.length; index++) {
        debugPrint("| Widget: ${pageStack[index].settings.name} order: $index");
      }

      debugPrint("└───────────────────────────────────────");
    }
  }

  @override
  void _onPop(Route route, Route previousRoute) {
    if (route.isPage()) {
      pageStack.removeLast();
    }
  }

  @override
  void _onPush(Route route, Route previousRoute) {
    if (route.isPage()) {
      pageStack.add(route);
    }
  }

  @override
  void _onReplace(Route newRoute, Route oldRoute) {
    if (oldRoute.isPage()) {
      pageStack.removeLast();
    }

    if (newRoute.isPage()) {
      pageStack.add(newRoute);
    }
  }
}
